"use strict"

class Customer {
    tagNumber = undefined
    name = undefined

    constructor(tn, name) {
        this.tagNumber = tn
        this.name = name
    }
}

class CustomerDatabase {
    list = []

    constructor() {
        // load a fake database here
    }

    // get the customer
    find(tn) {

    }
}

class Gate {

    animationStart = undefined
    gateArm = undefined // the actual gate
    gateStand = undefined // the vertical post
    messageBoard = undefined // the box where the messages will be put
    trafficLight = undefined // the box holding the stop/go light

    gateSpeed = 0.01
    state = 0
    gatePosition = 0

    constructor(svgContainer) {
        this.svg = svgContainer

        this.gateArm = document.createElementNS('http://www.w3.org/2000/svg', 'rect')
        this.gateArm.setAttributeNS(null, 'x', 40)
        this.gateArm.setAttributeNS(null, 'y', 530)
        this.gateArm.setAttributeNS(null, 'width', 400)
        this.gateArm.setAttributeNS(null, 'height', 20)
        this.gateArm.setAttributeNS(null, 'style', 'fill:rgb(0,0,244)')

        this.gateStand = document.createElementNS('http://www.w3.org/2000/svg', 'rect')
        this.gateStand.setAttributeNS(null, 'x', 30)
        this.gateStand.setAttributeNS(null, 'y', 200)
        this.gateStand.setAttributeNS(null, 'width', 20)
        this.gateStand.setAttributeNS(null, 'height', 400)

        let msgFrame = document.createElementNS('http://www.w3.org/2000/svg', 'rect')
        msgFrame.setAttributeNS(null, 'x', 30)
        msgFrame.setAttributeNS(null, 'y', 150)
        msgFrame.setAttributeNS(null, 'width', 300)
        msgFrame.setAttributeNS(null, 'height', 100)
        msgFrame.setAttributeNS(null, 'style', 'fill:white;stroke:black')

        let message = document.createElementNS('http://www.w3.org/2000/svg', 'text')
        //message.innerHTML = "blah"
        message.appendChild(document.createTextNode('Good Morning'))
        message.setAttribute('x', 40)
        message.setAttribute('y', 200)

        this.messageBoard = {
            frame: msgFrame,
            text: message
        }

        let lightBody = document.createElementNS('http://www.w3.org/2000/svg', 'rect')
        lightBody.setAttribute('x', 30)
        lightBody.setAttribute('y', 260)
        lightBody.setAttribute('width', 60)
        lightBody.setAttribute('height', 120)
        lightBody.setAttribute('style', 'fill:grey;stroke:black')

        let redLight = document.createElementNS('http://www.w3.org/2000/svg', 'circle')
        redLight.setAttribute('cx', 60)
        redLight.setAttribute('cy', 290)
        redLight.setAttribute('r', 20)
        redLight.setAttribute('style', 'fill:red')

        let greenLight = document.createElementNS('http://www.w3.org/2000/svg', 'circle')
        greenLight.setAttribute('cx', 60)
        greenLight.setAttribute('cy', 340)
        greenLight.setAttribute('r', 20)

        this.trafficLight = {
            body: lightBody,
            red: redLight,
            green: greenLight
        }        

        this.svg.appendChild(this.gateArm)
        this.svg.appendChild(this.gateStand)
        this.svg.appendChild(this.messageBoard.frame)
        this.svg.appendChild(this.messageBoard.text)
        this.svg.appendChild(this.trafficLight.body)
        this.svg.appendChild(this.trafficLight.red)
        this.svg.appendChild(this.trafficLight.green)
    }

    // Cause the gate to open for delay seconds.  Gate closes automatically at the end unless the user calls the close
    // function before.
    open(delay = 5) {        
        this.animationStart = undefined
        this.state = 1
        requestAnimationFrame((ts) => this.showGateAnimation(ts))        
    }

    showGateAnimation(timestamp) {
        if (!this.animationStart) {
            this.animationStart = timestamp
        }
        var stepTime = timestamp - this.animationStart
        if (this.state === 1) {
            this.gatePosition -= this.gateSpeed*stepTime

            if (this.gatePosition < -90) {
                this.gatePosition = -90
            }
            this.gateArm.setAttribute("transform", `rotate(${this.gatePosition} 40 535)`)
            if (this.gatePosition > -90) {
                requestAnimationFrame((ts) => this.showGateAnimation(ts))
            }
        }
        if (this.state === 2) {
            this.gatePosition += this.gateSpeed*stepTime

            if (this.gatePosition > 0) {
                this.gatePosition = 0
            }
            this.gateArm.setAttribute("transform", `rotate(${this.gatePosition} 40 535)`)
            if (this.gatePosition < 0) {
                requestAnimationFrame((ts) => this.showGateAnimation(ts))
            }

        }
        
    }

    // Close the gate now.  Could be used after the car has passed the sensor
    close() {
        this.animationStart = undefined
        this.state = 2
        requestAnimationFrame((ts) => this.showGateAnimation(ts))
    }

    // display a message to the driver on the screen
    showMessage(msg) {
        //console.log(this.messageBoard.textContext)
        this.messageBoard.text.innerHTML = msg
    }

    // show a large colored light to driver
    setLight(color) {
        if (color === "red") {
            this.trafficLight.red.setAttribute('style', 'fill:red')
            this.trafficLight.green.setAttribute('style', 'fill:black')
        }
        else if (color === "green") {
            this.trafficLight.red.setAttribute('style', 'fill:black')
            this.trafficLight.green.setAttribute('style', 'fill:lime')

        }
        else {
            this.trafficLight.red.setAttribute('style', 'fill:black')
            this.trafficLight.green.setAttribute('style', 'fill:black')

        }
    }
}
